package lab.game;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

import javafx.geometry.Dimension2D;
import javafx.geometry.Point2D;
import javafx.scene.canvas.GraphicsContext;

public class Scene {

	private Dimension2D size;
	private List<DrawableSimulable> sceneEntitites;
	private List<DrawableSimulable> entititesToAdd = new ArrayList<>();
	private List<DrawableSimulable> entititesToRemove = new ArrayList<>();

	private Boat boat;
	private Rock rock;
	private Background background;

	public Scene(double width, double height, int numberOfMonsters) {
		size = new Dimension2D(width, height);
		sceneEntitites = new ArrayList<>();
		background = new Background(this);
		rock = new Rock(this, new Point2D(300, 300), new Dimension2D(30, 50));
		boat = new Boat(this, new Point2D(20, 200));
		sceneEntitites.add(background);
		sceneEntitites.add(rock);
		sceneEntitites.add(boat);
		for (int i = 0; i < numberOfMonsters; i++) {
			sceneEntitites.add(new LochNess(this));
		}
	}

	public Dimension2D getSize() {
		return size;
	}

	public void draw(GraphicsContext gc) {
		for (DrawableSimulable entity : sceneEntitites) {
			entity.draw(gc);
		}
	}

	public void simulate(double deltaTime) {
		for (DrawableSimulable drawableSimulable : sceneEntitites) {
			drawableSimulable.simulate(deltaTime);
		}
		for (int i = 0; i < sceneEntitites.size(); i++) {
			if (sceneEntitites.get(i) instanceof Collisionable c1) {
				for (int j = i + 1; j < sceneEntitites.size(); j++) {
					if (sceneEntitites.get(j) instanceof Collisionable c2) {
						if (c1.intersect(c2.getBoundingBox())) {
							c1.hitBy(c2);
							c2.hitBy(c1);
						}
					}
				}
			}
		}
		sceneEntitites.removeAll(entititesToRemove);
		sceneEntitites.addAll(entititesToAdd);
		entititesToAdd.clear();
		entititesToRemove.clear();
		sceneEntitites.sort(Comparator.comparing(DrawableSimulable::getZIndex));
	}

	public Boat getBoat() {
		return boat;
	}

	public Rock getRock() {
		return rock;
	}

	public Background getBackground() {
		return background;
	}
	
	public void add(DrawableSimulable entity) {
		entititesToAdd.add(entity);
	}
	public void remove(DrawableSimulable entity) {
		entititesToRemove.add(entity);
	}

}
